% hfoExaminer(): a part of MoBILAB customized for maximally intuitive 
%                EEG event marking. DO NOT USE WITH MOBILAB otherwise paths
%                should conflict.

% History
% 03/27/13 ver 1.1 by Makoto and Alejandro. Mouse scroll. Independent of Mobilab.
% 12/26/12 ver 1.0 by Alejandro and Makoto. Created.

% Copyright (C) 2013, Makoto Miyakoshi JSPS/SCCN,INC,UCSD
%                     Alejandro Ojeda  SCCN,INC,UCSD
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

%% Creates an object from the class hfoExaminerHandle
%
% hfoExaminerHandle object controls the application hfoExaminer
%
% Authors: Alejandro Ojeda and Makoto Miyakoshi
% Swartz Center for Computational Neuroscience, Institute for Neural Computation, University
% of California San Diego; Japan Society for the Promotion of Science.

classdef hfoExaminer < eegplotNGHandle
    properties
        newEventType
        eventMarker
        markerType
        markerSize
        markerEdgeLineWidth
        markerEdgeColor
        cursorType
        cursorSize
        cursorLineWidth
        markerSelectErrorMarginRate
    end
    methods
        function obj = hfoExaminer(EEG,onButtonDownCallback,onMouseMoveCallback)
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% shortcut for mouse function %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if nargin < 2, onButtonDownCallback = 'axes_ButtonDownFcn';end
            if nargin < 3, onMouseMoveCallback = 'onMouseMoveCallback';end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% channel status check %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if isempty(EEG.chanlocs)
                disp('Channel information is Empty. Generating incremental indices.')
                for n = 1:EEG.nbchan
                    EEG.chanlocs(1,n).labels = num2str(n);
                    EEG.chanlocs(1,n).index  = n;
                end
                
                assignin('base','EEG',EEG);
            end            
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% event status check %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%
            if isempty(EEG.event)
                disp('Event information is Empty. Generating an initial event.')
                EEG.event.type       = 'start';
                EEG.event.latency    = 1;
                EEG.event.init_index = 1;
                EEG.event.init_time  = 1/EEG.srate;
                EEG.event.urevent = 1;

                EEG.urevent = [];
                EEG.urevent.type       = 'start';
                EEG.urevent.latency    = 1;
                EEG.urevent.init_index = 1;
                EEG.urevent.init_time  = 1/EEG.srate;
                
                assignin('base','EEG',EEG);
            end
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% inherit eegplotNGHandle %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%            
            obj@eegplotNGHandle(EEG);
            obj.step = 100;
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% super small Pointer %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%
            %
            %      1 1 1
            %      1 0 1
            %      1 1 1
            %
            %  the hot spot == 0
            pointerShape           = NaN(16,16);
            pointerShape(1, 1:3)   = 1;
            pointerShape(2, [1 3]) = 1;
            pointerShape(3, 1:3)   = 1;
            pointerHotSpot         = [2,2];
            set(obj.figureHandle,'Pointer','custom', 'PointerShapeCData', pointerShape, 'PointerShapeHotSpot', pointerHotSpot)
            
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%% default value setting %%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            obj.markerType                  = 'diamond';
            obj.markerSize                  = 14;
            obj.markerEdgeLineWidth         = 1;
            obj.markerEdgeColor             = [0,0,0];
            obj.cursorType                  = 'x';
            obj.cursorSize                  = 120;
            obj.cursorLineWidth             = 2;
            obj.markerSelectErrorMarginRate = 0.4;
            
            
            
            delete(obj.makeSegmentHandle);
            obj.newEventType = 'HFO'; 
            obj.onscreenDisplay = false;
            set(obj.figureHandle,'WindowButtonMotionFcn', {onMouseMoveCallback,obj},'Name','hfoExaminer(EEG)');

            if isfield(EEG.event,'channel')
                obj.streamHandle.event = event;
                channel = {EEG.event.channel};
                latency = cell2mat({EEG.event.latency});
                type = {EEG.event.type};
                indHFO = ismember(type,obj.newEventType);
                for it=find(indHFO)
                    if ismac, [~,uuid] = system('uuidgen'); else uuid =  java.util.UUID.randomUUID;end
                    type{it} = [type{it} '_' char(uuid) '_' num2str(channel{it})];
                end
                
                obj.streamHandle.event = obj.streamHandle.event.addEvent(latency(indHFO),type(indHFO));
            end
            obj.plotThisTimeStamp(obj.nowCursor);
        end
        %%

        function delete(obj)
            type    = obj.newEventType;
            label   = obj.streamHandle.event.label;
            % indices = cell2mat(strfind(label,type));
            indices = strncmp(label, type, 3);
            % indices(indices==0) = [];
            latency = obj.streamHandle.event.latencyInFrame(indices);
            type = obj.streamHandle.event.label(indices);
            if isempty(latency), return;end
            
            channel = zeros(length(latency),1);
            for it=1:length(latency)
                ind = find(type{it} == '_');
                channel(it) = str2double(type{it}(ind(end)+1:end));
            end
                        
            EEG = evalin('base','EEG');

            I = ismember({EEG.event.type},obj.newEventType);
            if sum(I) == length(latency) && all(cell2mat({EEG.event(I).latency}) == latency), return;end
            EEG = pop_editeventvals(EEG,'delete',find(I));
            
            disp('Saving new events...');
            EEG = eeg_addnewevents(EEG, {latency}, {obj.newEventType},{'channel'},{channel});
            assignin('base','EEG',EEG);
            eeglab redraw
            disp('Done!!!')
        end
        %%
        
        function obj = changeSettings(obj)
            sg = sign(double(obj.speed<1));
            sg(sg==0) = -1;
            speed1 = [num2str(-sg*obj.speed^(-sg)) 'x'];
            speed2 = [1/5 1/4 1/3 1/2 1 2 3 4 5];
            
            prefObj = [...
                PropertyGridField('gain',obj.gain,'DisplayName','Channel gain','Description','')...
                PropertyGridField('step',obj.step,'DisplayName','Step','Description','onMouseWheelMove step')...
                PropertyGridField('channels',obj.channelIndex,'DisplayName','Channels to plot','Description','This field accept matlab code returning a subset of channels, for instance use: ''setdiff(1:10,[3 5 7])'' to plot channels from 1 to 10 excepting 3, 5, and 7.')...
                PropertyGridField('speed',speed1,'Type',PropertyType('char','row',{'-5x','-4x','-3x','-2x','1x','2x','3x','4x','5x'}),'DisplayName','Speed','Description','Speed of the play mode.')...
                PropertyGridField('windowWidth',obj.windowWidth,'DisplayName','Window width','Description','Size of the segment to plot in seconds.')...
                PropertyGridField('normalizeFlag',obj.normalizeFlag,'DisplayName','Normalize channels','Description','Divides each channels by its standard deviation within the segment to plot.')...
                PropertyGridField('showChannelNumber',obj.showChannelNumber,'DisplayName','Show channel number or label','Description','')...
                PropertyGridField('onscreenDisplay',obj.onscreenDisplay,'DisplayName','Show events','Description','')...
                PropertyGridField('newEventType',obj.newEventType,'DisplayName','Type of new events','Description','')...
                PropertyGridField('labels', obj.streamHandle.event.uniqueLabel,'DisplayName','Show only a subset of events','Description','')...
                PropertyGridField('colormap',obj.colormap,'Type',PropertyType('char','row',{'lines','eegplot'}),'DisplayName','Colormap','Description','')...
                PropertyGridField('markerType',obj.markerType,                                  'DisplayName','marker type', 'Type',PropertyType('char','row',{'+', 'o', '*', '.', 'x', 'square', 'diamond', '^', 'v', '>', '<', 'pentagram', 'hexagram'}),'Description','Select event marker type.')...
                PropertyGridField('markerSize',obj.markerSize,                                  'DisplayName','marker size',                  'Description','Specify the size of the event marker.')...
                PropertyGridField('markerEdgeLineWidth',obj.markerEdgeLineWidth,                'DisplayName','marker edge line width',       'Description','Specify the edge line width of the event marker.')...
                PropertyGridField('markerEdgeColor', obj.markerEdgeColor,                       'DisplayName','marker edge color',            'Description','Specify the edge color of the event marker.')...
                PropertyGridField('cursorType',obj.cursorType,                                  'DisplayName','cursor type', 'Type',PropertyType('char','row',{'+', 'o', '*', '.', 'x', 'square', 'diamond', '^', 'v', '>', '<', 'pentagram', 'hexagram'}),'Description','Select cursor type.')...
                PropertyGridField('cursorSize',obj.cursorSize,                                  'DisplayName','cursor size',                  'Description','Specify the size of the cursor.')...
                PropertyGridField('cursorLineWidth',obj.cursorLineWidth,                        'DisplayName','cursor line width',            'Description','Specify the line width of the cursor.')...
                PropertyGridField('markerSelectErrorMarginRate',obj.markerSelectErrorMarginRate,'DisplayName','error margin in marker deletion','Description','Specify the error margin [%] when selecting markers for deletion. The rate is against a page width.')...
                ];
            
            % create figure
            f = figure('MenuBar','none','Name','Preferences','NumberTitle', 'off','Toolbar', 'none');
            position = get(f,'position');
            set(f,'position',[position(1:2) 385 424]);
            g = PropertyGrid(f,'Properties', prefObj,'Position', [0 0 1 1]);
            uiwait(f); % wait for figure to close
            val = g.GetPropertyValues();
            
            obj.eventObj = event;
            if ~isempty(val.labels)
                for it=1:length(val.labels)
                    latency = obj.streamHandle.event.getLatencyForEventLabel(val.labels{it});
                    obj.eventObj = obj.eventObj.addEvent(latency,val.labels{it});
                end
            end

            obj.gain              = val.gain;
            obj.step              = val.step;
            obj.speed             = speed2(ismember({'-5x','-4x','-3x','-2x','1x','2x','3x','4x','5x'},val.speed));
            obj.windowWidth       = val.windowWidth;
            obj.normalizeFlag     = val.normalizeFlag;
            obj.showChannelNumber = val.showChannelNumber;
            obj.channelIndex      = val.channels;
            obj.onscreenDisplay   = val.onscreenDisplay;
            obj.newEventType      = val.newEventType;
            obj.changeColormap(val.colormap);
            obj.markerType        = val.markerType;
            obj.markerSize        = val.markerSize;
            obj.markerEdgeLineWidth = val.markerEdgeLineWidth;
            obj.markerEdgeColor   = val.markerEdgeColor;
            obj.cursorType        = val.cursorType;
            obj.cursorSize        = val.cursorSize;
            obj.cursorLineWidth   = val.cursorLineWidth;
            obj.markerSelectErrorMarginRate = val.markerSelectErrorMarginRate;
            
            figure(obj.figureHandle);
            obj.nowCursor = obj.streamHandle.timeStamp(obj.timeIndex(1)) + obj.windowWidth/2;
            obj.createGraphicObjects(obj.nowCursor);
        end
        %%
        
        function plotThisTimeStamp(obj,nowCursor)
            obj.onscreenDisplay = 0;
            plotThisTimeStamp@eegplotNGHandle(obj,nowCursor);
            
            %- plot diamonts (to be modified)
            % reading the channels + labels of old HFO events
            % these labels do not have uuids
            
            if isempty(obj.newEventType), return;end
            label = obj.streamHandle.event.label;
            latency = obj.streamHandle.event.latencyInFrame;
            % ind = cell2mat(strfind(label,obj.newEventType));
            ind = strncmp(label, obj.newEventType, 3);
            if ~any(ind), return;end
            
            latency(~ind) = [];
            label(~ind) = [];
            
            n = sum(ind);
            channel = zeros(n,1);
            for it=1:n
                loc = find(label{it}=='_');
                channel(it) = str2double(label{it}(loc(end)+1:end)); 
            end
            
            % if the events are inside the window then plot them       
            window = [obj.nowCursor - obj.windowWidth/2 obj.nowCursor + obj.windowWidth/2];
            latencyEventsInSeconds = obj.streamHandle.timeStamp( latency );
            loc = find(latencyEventsInSeconds > window(1)  & latencyEventsInSeconds < window(2));

            time = obj.streamHandle.timeStamp( latency(loc) );
            channel = channel(loc);
            label = label(loc);
            hold(obj.axesHandle,'on');
            try delete(obj.eventMarker);end %#ok
            obj.eventMarker = zeros(length(loc),1);
            %set(obj.figureHandle,'CurrentAxes',obj.axesHandle)
            
            for it=1:length(loc)
                tmpTime = get(obj.gObjHandle(channel(it)),'XData');
                [~,locTime] = min(abs(tmpTime - time(it)));
                voltage = get(obj.gObjHandle(channel(it)),'YData');
                
                lineColor = get(obj.gObjHandle(channel(it)),'Color');
                
                obj.eventMarker(it) = obj.plotEventMarker(tmpTime(locTime),voltage(locTime),label(it), lineColor, channel(it));
            end
            %set(obj.figureHandle,'CurrentAxes',obj.axesHandle)
            hold(obj.axesHandle,'off');
        end
        %%
        
        function markerHandle = plotEventMarker(obj,timeStamp,value,eventLabel, lineColor, latestChannel)
            markerHandle = plot(timeStamp, value, obj.markerType, 'MarkerSize', obj.markerSize, 'linewidth', obj.markerEdgeLineWidth, 'MarkerEdgeColor', obj.markerEdgeColor, 'MarkerFaceColor', lineColor, 'tag', ['diamond_ch' num2str(latestChannel)],'userdata',eventLabel,'Parent',obj.axesHandle, 'HitTest', 'off');
        end
        %%
        
        function onMouseWheelMove(obj,~,eventObj)
            step = -10*obj.step*(eventObj.VerticalScrollCount*eventObj.VerticalScrollAmount)/obj.streamHandle.samplingRate;
            plotStep(obj,step);
        end
     end
end